#!/bin/sh -e
# this program compares timings to compile and run pystone.py via cython and gcc/clang/tcc.

export LANG=C.UTF-8	# . for floating point in e.g. time output

git clean -fdX


CFLAGS="-pthread -DNDEBUG -fwrapv -O0 -Wall -Wstrict-prototypes -fno-strict-aliasing -Wdate-time -fPIC -I/usr/include/python2.7"

# compile <name>
compile() {
	name=$1; shift
	$CC $CFLAGS "$@" -c $name.c -o $name.o
}
	
# link <name>
link() {
	name=$1; shift
	$CC $CFLAGS "$@" -shared $name.o -o $name.so
}

# compileandlink <name>
compileandlink() {
	name=$1; shift
	$CC $CFLAGS "$@" -shared $name.c -o $name.so
}

# pystone2bench <subj> converst pystone output into go benchmark format
pystone2bench() {
	subj=$1
	tail -1 |sed -e \
	"s|^This machine benchmarks at \([0-9.e+]\+\) pystones/second$|Benchmarkpystone/${subj} 1 \1 pystone/s|"
}

N=10

# python: pycompile + pyimport
for i in `seq $N`; do
	rm -f pystone.pyc
	export TIMEFORMAT="Benchmarkpycompile+pyimport 1 %R s/op"
	time python -c 'import pystone'
done

# python: pyimport with .pyc already being there
python -c 'import pystone'
for i in `seq $N`; do
	export TIMEFORMAT="Benchmarkpyimport 1 %R s/op"
	time python -c 'import pystone'
done

# cython pystone.py
for i in `seq $N`; do
	cp pystone.py pystone_pypyx.py # to be sure that 'import pystone_pypyx' does not pick up py code
	export TIMEFORMAT="Benchmarkcython/py 1 %R s/op"
	time cython -2 pystone_pypyx.py # -> pystone_pypyx.c
	#wc -l pystone_pypyx.{py,c}
	rm pystone_pypyx.py
done

# cython pystone_pyx.pyx
for i in `seq $N`; do
	export TIMEFORMAT="Benchmarkcython/pyx 1 %R s/op"
	time cython -2 pystone_pyx.pyx # -> pystone_pyx.c
done


# pystone.py
for i in `seq $N`; do
	python pystone.py |pystone2bench py
done

#for CC in gcc clang tcc; do	# clang is close to gcc - don't spend time on it
for CC in gcc tcc; do
	echo "CC=$CC"
	rm -f pystone_pypyx.{o,so}

	# don't spend time measuring compile and link separatelly: we know from
	# past benchmarks that most of the time is in compile.
	#for i in `seq $N`; do
	#	export TIMEFORMAT="Benchmark${CC}/compile/py 1 %R s/op"
	#	time compile pystone_pypyx
	#done
	#
	#for i in `seq $N`; do
	#	export TIMEFORMAT="Benchmark${CC}/link/py 1 %R s/op"
	#	time link pystone_pypyx
	#done

	for i in `seq $N`; do
		export TIMEFORMAT="Benchmark${CC}/compile+link/py 1 %R s/op"
		time compileandlink pystone_pypyx
	done

	# pystone.py (compiled via cython)
	for i in `seq $N`; do
		python -c 'import pystone_pypyx as _; _.main()' |pystone2bench py/$CC
	done

	# tcc -O2 does not add anything (tcc accepts -Ox only for compatibility)
	if [ $CC != tcc ]; then
		for i in `seq $N`; do
			export TIMEFORMAT="Benchmark${CC}/compile+linkO2/py 1 %R s/op"
			time compileandlink pystone_pypyx "-O2"
		done

		for i in `seq $N`; do
			python -c 'import pystone_pypyx as _; _.main()' |pystone2bench py/${CC}O2
		done
	fi


	for i in `seq $N`; do
		export TIMEFORMAT="Benchmark${CC}/compile+link/pyx 1 %R s/op"
		time compileandlink pystone_pyx
	done

	# pystone_pyx.pyx
	for i in `seq $N`; do
		python -c 'import pystone_pyx as _; _.main()' |pystone2bench pyx/$CC
	done

	# tcc -O2 ...
	if [ $CC != tcc ]; then
		for i in `seq $N`; do
			export TIMEFORMAT="Benchmark${CC}/compile+linkO2/pyx 1 %R s/op"
			time compileandlink pystone_pyx "-O2"
		done

		for i in `seq $N`; do
			python -c 'import pystone_pyx as _; _.main()' |pystone2bench pyx/${CC}O2
		done
	fi
done
